/*
 * __arduino_on_xmos_test.xc
 *
 *  Created on: 17. jan. 2018
 *      Author: teig
 */
#ifdef COMMENT_BLOCK
// XMOS Programming Guide. Publication Date: 2014/9/18

// The pins on the device are accessed via the hardware-response ports.
// The port is responsible for driving output on the pins or sampling input data.
// Different ports have different widths: there are 1-bit, 4-bit, 8-bit, 16-bit and 32-bit ports.
// An n-bit port will simultaneously drive or sample n bits at once.
// In current devices, each tile has 29 ports.

// Port width  Number of ports  Port names
// 1           16               1A, 1B, 1C, 1D, 1E, 1F, 1G, 1H, 1I, 1J, 1K, 1L, 1M, 1N, 1O ,1P
// 4           6                4A, 4B, 4C, 4D, 4E ,4F
// 8           4                8A, 8B, 8C ,8D
// 16          2                16A, 16B
// 32          1                32A

// All ports are clocked - they are attached to a clock block in the device to control reading and writing
// from the port. A clock block provides a regular clock signal to the port.

// Each port has a register called the shift register within it that holds either data to output or data
// just input depending on whether the port is in input or output mode. At each clock tick, the port
// samples the external pins into the shift register or drives the external pins based on the contents
// of the shift register. When a program inputs or outputs to a port it is actually reads or writes
// the shift register.

// There are six clock blocks per tile. Any port can be connected to any of these six clock blocks.
// Each port can be set to one of two modes:
// - Divide             The clock runs at a rate which is an integer divide of the core clock rate of the
//                      chip (e.g. a divide of 500MHz for a 500MHz part).
// - Externally driven  The clock runs at a rate governed by an port input.

// By default, all ports are connected to clock block 0 which is designated the reference
// clock block and always runs at 100MHz.

// All ports must be declared as global variables, and no two ports may be initialized with the same port
// identifier. After initialization, a port may not be assigned to. Passing a port to a function is allowed
// as long as the port does not appear in more than one of a functions arguments, which would create an
// illegal alias

#endif

#include <xs1.h>
#include <stdint.h> // uint8_t


// -------------------------------
// --       LIBRARY CODE        --
// -------------------------------

// BOOLEAN #include <stdbool.h> if C99
typedef enum {false,true} bool;     // 0,1
typedef enum {LOW,HIGH}   pinval_t; // 0,1


#define PIN_1A      out port pin_out_1A = XS1_PORT_1A // Not used
//      PIN_1A do { out port pin_out_1A = XS1_PORT_1A; } while (0) // Not allowed

//      https://www.arduino.cc/reference/en/language/functions/digital-io/pinmode/
//      https://garretlab.web.fc2.com/en/arduino/inside/arduino/wiring_digital.c/pinMode.html
// void pinMode(uint8_t pin, uint8_t mode) // Arduino
//      pinMode(13,OUTPUT) // Arduino
#define PINMODE_OUTPUT_pin_out_1X(num) out port pin_out_##num = XS1_PORT_##num // Usage global only
//
//      pinMode(12,INPUT) // Arduino
#define PINMODE_INPUT_pin_in_1X(num)  in  port pin_in_##num  = XS1_PORT_##num // Usage global only
//
//     pinMode(12,INPUT_PULLUP) // Arduino, not implemented here


// https://garretlab.web.fc2.com/en/arduino/inside/arduino/wiring_digital.c/digitalWrite.html
// https://www.arduino.cc/reference/en/language/functions/digital-io/digitalwrite/
//
// void digitalWrite(uint8_t pin, uint8_t val) // Arduino
void digitalWrite (out port pin, const pinval_t pinval) {
    pin <: pinval;
}


// https://garretlab.web.fc2.com/en/arduino/inside/arduino/wiring_digital.c/digitalRead.html
// https://www.arduino.cc/reference/en/language/functions/digital-io/digitalread/
//
// int digitalRead(uint8_t pin) // Arduino
pinval_t digitalRead (in port pin) {
    pinval_t pinval;
    pin :> pinval;
    return pinval;
}


// -------------------------------
// --        USER CODE          --
// -------------------------------

#define PIN_13_LED    1A
#define PIN_07_BUTTON 1B

//#define COMPILES_FINE

#ifdef COMPILES_FINE
 // pinMode(13,OUTPUT) // Arduino
    PINMODE_OUTPUT_pin_out_1X(1A); // "pin_out_1A" created

 // pinMode(12,INPUT) // Arduino
    PINMODE_INPUT_pin_in_1X (1B); // "pin_in_1B"  created
#else
    PINMODE_OUTPUT_pin_out_1X (PIN_13_LED);    // error: use of undeclared identifer `XS1_PORT_PIN_13_LED'
    PINMODE_INPUT_pin_in_1X   (PIN_07_BUTTON); // error: use of undeclared identifer `XS1_PORT_PIN_07_BUTTON'
#endif

int main() {
    #ifdef COMMENT_BLOCK
    // Arduino example at https://www.arduino.cc/reference/en/language/functions/digital-io/digitalread/
    //
    // int ledPin = 13;   // LED connected to digital pin 13
    // int inPin = 7;     // pushbutton connected to digital pin 7
    // int val = 0;       // variable to store the read value
    //
    // void setup()
    // {
    //   pinMode(ledPin, OUTPUT);      // sets the digital pin 13 as output
    //   pinMode(inPin, INPUT);        // sets the digital pin 7 as input
    // }
    //
    // void loop()
    // {
    //   val = digitalRead(inPin);     // read the input pin
    //   digitalWrite(ledPin, val);    // sets the LED to the button's value
    // }
    #endif

    pinval_t pinval;
                                       // In __arduino_on_xmos_test.xc.o:
    pinval = digitalRead (pin_in_1B);  // note: object used here
                                       // error: call to `digitalWrite' in `main' makes alias of global 'pin_out_1A'
    digitalWrite (pin_out_1A, pinval); // error: call to `digitalRead' in `main' makes alias of global 'pin_in_1B'

    // digitalWrite (pin_out_1A, digitalRead (pin_in_1B)); // One-liner2

    return 0;
}
